/*****************************************************************************
 * Copyright (c) 2007, 2009 Rowley Associates Limited.                       *
 *                                                                           *
 * This file may be distributed under the terms of the License Agreement     *
 * provided with this software.                                              *
 *                                                                           *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE   *
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. *
 *****************************************************************************/

/*****************************************************************************
 *                           Preprocessor Definitions
 *                           ------------------------
 *
 * STARTUP_FROM_RESET
 *
 *   If defined, the program will startup from power-on/reset. If not defined
 *   the program will just loop endlessly from power-on/reset.
 *
 *   This definition is not defined by default on this target because the
 *   debugger is unable to reset this target and maintain control of it over the
 *   JTAG interface. The advantage of doing this is that it allows the debugger
 *   to reset the CPU and run programs from a known reset CPU state on each run.
 *   It also acts as a safety net if you accidently download a program in FLASH
 *   that crashes and prevents the debugger from taking control over JTAG
 *   rendering the target unusable over JTAG. The obvious disadvantage of doing
 *   this is that your application will not startup without the debugger.
 *
 *   We advise that on this target you keep STARTUP_FROM_RESET undefined whilst
 *   you are developing and only define STARTUP_FROM_RESET when development is
 *   complete.
 *
 * __NO_SYSTEM_INIT
 *
 *   If defined, the SystemInit() function will NOT be called. By default SystemInit()
 *   is called after reset to enable the clocks and memories to be initialised 
 *   prior to any C startup initialisation.
 *
 * __FLASH_BUILD
 *
 *   If not defined then the exception vectors are located to the address defined
 *   by the linker symbol _vectors.
 *
 *****************************************************************************/

  .global reset_handler

  .section .vectors, "ax"
  .code 16
  .align 0
  .global _vectors
  .syntax unified

.macro DEFAULT_ISR_HANDLER name=
  .thumb_func
  .weak \name
\name:
1: b 1b /* endless loop */
.endm

_vectors:
  .word __stack_end__
#ifdef STARTUP_FROM_RESET
  .word reset_handler
#else
  .word reset_wait
#endif /* STARTUP_FROM_RESET */
  .word NMI_Handler
  .word HardFault_Handler
  .word MemManage_Handler 
  .word BusFault_Handler
  .word UsageFault_Handler
  .word 0 // Reserved
  .word 0 // Reserved
  .word 0 // Reserved
  .word 0 // Reserved
  .word SVC_Handler
  .word DebugMon_Handler
  .word 0 // Reserved
  .word PendSV_Handler
  .word SysTick_Handler 
  .word WWDG_IRQHandler
  .word PVD_IRQHandler
  .word TAMPER_IRQHandler
  .word RTC_IRQHandler
  .word FLASH_IRQHandler
  .word RCC_IRQHandler
  .word EXTI0_IRQHandler
  .word EXTI1_IRQHandler
  .word EXTI2_IRQHandler
  .word EXTI3_IRQHandler
  .word EXTI4_IRQHandler
  .word DMA1_Channel1_IRQHandler
  .word DMA1_Channel2_IRQHandler
  .word DMA1_Channel3_IRQHandler
  .word DMA1_Channel4_IRQHandler
  .word DMA1_Channel5_IRQHandler
  .word DMA1_Channel6_IRQHandler
  .word DMA1_Channel7_IRQHandler
  .word ADC1_2_IRQHandler
#ifdef __TARGET_CL
  .word CAN1_TX_IRQHandler
  .word CAN1_RX0_IRQHandler
#else
  .word USB_HP_CAN1_TX_IRQHandler
  .word USB_LP_CAN1_RX0_IRQHandler
#endif
  .word CAN1_RX1_IRQHandler
  .word CAN1_SCE_IRQHandler
  .word EXTI9_5_IRQHandler
  .word TIM1_BRK_IRQHandler
  .word TIM1_UP_IRQHandler
  .word TIM1_TRG_COM_IRQHandler
  .word TIM1_CC_IRQHandler
  .word TIM2_IRQHandler
  .word TIM3_IRQHandler
  .word TIM4_IRQHandler  
  .word I2C1_EV_IRQHandler
  .word I2C1_ER_IRQHandler
  .word I2C2_EV_IRQHandler
  .word I2C2_ER_IRQHandler
  .word SPI1_IRQHandler
  .word SPI2_IRQHandler
  .word USART1_IRQHandler
  .word USART2_IRQHandler
  .word USART3_IRQHandler
  .word EXTI15_10_IRQHandler
  .word RTCAlarm_IRQHandler
#ifdef __TARGET_CL
  .word OTG_FS_WKUP_IRQHandler
#else
  .word USBWakeUp_IRQHandler
#endif
  .word TIM8_BRK_IRQHandler
  .word TIM8_UP_IRQHandler
  .word TIM8_TRG_COM_IRQHandler
  .word TIM8_CC_IRQHandler
  .word ADC3_IRQHandler
  .word FSMC_IRQHandler
  .word SDIO_IRQHandler
  .word TIM5_IRQHandler
  .word SPI3_IRQHandler
  .word UART4_IRQHandler
  .word UART5_IRQHandler
  .word TIM6_IRQHandler
  .word TIM7_IRQHandler
  .word DMA2_Channel1_IRQHandler
  .word DMA2_Channel2_IRQHandler
  .word DMA2_Channel3_IRQHandler
#ifdef __TARGET_CL
  .word DMA2_Channel4_IRQHandler
  .word DMA2_Channel5_IRQHandler
  .word ETH_IRQHandler
  .word ETH_WKUP_IRQHandler
  .word CAN2_TX_IRQHandler
  .word CAN2_RX0_IRQHandler
  .word CAN2_RX1_IRQHandler
  .word CAN2_SCE_IRQHandler
  .word OTG_FS_IRQHandler
#else
  .word DMA2_Channel4_5_IRQHandler
#endif

  .section .init, "ax"
  .thumb_func

  reset_handler:

#ifndef __NO_SYSTEM_INIT
  ldr sp, =__RAM_segment_end__
  bl SystemInit
#endif

#ifndef __FLASH_BUILD
  /* If this is a RAM build, configure vector table offset register to point 
     to the RAM vector table. */
  ldr r0, =0xE000ED08
  ldr r1, =_vectors
  str r1, [r0]
#endif
  b _start

#ifndef __NO_SYSTEM_INIT
  .thumb_func
  .weak SystemInit
SystemInit:
  bx lr
#endif

DEFAULT_ISR_HANDLER NMI_Handler
DEFAULT_ISR_HANDLER HardFault_Handler 
DEFAULT_ISR_HANDLER MemManage_Handler 
DEFAULT_ISR_HANDLER BusFault_Handler 
DEFAULT_ISR_HANDLER UsageFault_Handler 
DEFAULT_ISR_HANDLER SVC_Handler 
DEFAULT_ISR_HANDLER DebugMon_Handler  
DEFAULT_ISR_HANDLER PendSV_Handler
DEFAULT_ISR_HANDLER SysTick_Handler 
DEFAULT_ISR_HANDLER WWDG_IRQHandler 
DEFAULT_ISR_HANDLER PVD_IRQHandler 
DEFAULT_ISR_HANDLER TAMPER_IRQHandler 
DEFAULT_ISR_HANDLER RTC_IRQHandler 
DEFAULT_ISR_HANDLER FLASH_IRQHandler 
DEFAULT_ISR_HANDLER RCC_IRQHandler 
DEFAULT_ISR_HANDLER EXTI0_IRQHandler 
DEFAULT_ISR_HANDLER EXTI1_IRQHandler 
DEFAULT_ISR_HANDLER EXTI2_IRQHandler 
DEFAULT_ISR_HANDLER EXTI3_IRQHandler 
DEFAULT_ISR_HANDLER EXTI4_IRQHandler 
DEFAULT_ISR_HANDLER DMA1_Channel1_IRQHandler 
DEFAULT_ISR_HANDLER DMA1_Channel2_IRQHandler 
DEFAULT_ISR_HANDLER DMA1_Channel3_IRQHandler 
DEFAULT_ISR_HANDLER DMA1_Channel4_IRQHandler 
DEFAULT_ISR_HANDLER DMA1_Channel5_IRQHandler 
DEFAULT_ISR_HANDLER DMA1_Channel6_IRQHandler 
DEFAULT_ISR_HANDLER DMA1_Channel7_IRQHandler 
DEFAULT_ISR_HANDLER ADC1_2_IRQHandler 
#ifdef __TARGET_CL
DEFAULT_ISR_HANDLER CAN1_TX_IRQHandler 
DEFAULT_ISR_HANDLER CAN1_RX0_IRQHandler
#else
DEFAULT_ISR_HANDLER USB_HP_CAN1_TX_IRQHandler 
DEFAULT_ISR_HANDLER USB_LP_CAN1_RX0_IRQHandler 
#endif
DEFAULT_ISR_HANDLER CAN1_RX1_IRQHandler 
DEFAULT_ISR_HANDLER CAN1_SCE_IRQHandler 
DEFAULT_ISR_HANDLER EXTI9_5_IRQHandler 
DEFAULT_ISR_HANDLER TIM1_BRK_IRQHandler 
DEFAULT_ISR_HANDLER TIM1_UP_IRQHandler 
DEFAULT_ISR_HANDLER TIM1_TRG_COM_IRQHandler 
DEFAULT_ISR_HANDLER TIM1_CC_IRQHandler 
DEFAULT_ISR_HANDLER TIM2_IRQHandler 
DEFAULT_ISR_HANDLER TIM3_IRQHandler 
DEFAULT_ISR_HANDLER TIM4_IRQHandler 
DEFAULT_ISR_HANDLER I2C1_EV_IRQHandler 
DEFAULT_ISR_HANDLER I2C1_ER_IRQHandler 
DEFAULT_ISR_HANDLER I2C2_EV_IRQHandler 
DEFAULT_ISR_HANDLER I2C2_ER_IRQHandler 
DEFAULT_ISR_HANDLER SPI1_IRQHandler 
DEFAULT_ISR_HANDLER SPI2_IRQHandler 
DEFAULT_ISR_HANDLER USART1_IRQHandler 
DEFAULT_ISR_HANDLER USART2_IRQHandler 
DEFAULT_ISR_HANDLER USART3_IRQHandler 
DEFAULT_ISR_HANDLER EXTI15_10_IRQHandler 
DEFAULT_ISR_HANDLER RTCAlarm_IRQHandler 
#ifdef __TARGET_CL
DEFAULT_ISR_HANDLER OTG_FS_WKUP_IRQHandler
#else
DEFAULT_ISR_HANDLER USBWakeUp_IRQHandler 
#endif
DEFAULT_ISR_HANDLER TIM8_BRK_IRQHandler
DEFAULT_ISR_HANDLER TIM8_UP_IRQHandler
DEFAULT_ISR_HANDLER TIM8_TRG_COM_IRQHandler
DEFAULT_ISR_HANDLER TIM8_CC_IRQHandler
DEFAULT_ISR_HANDLER ADC3_IRQHandler
DEFAULT_ISR_HANDLER FSMC_IRQHandler
DEFAULT_ISR_HANDLER SDIO_IRQHandler
DEFAULT_ISR_HANDLER TIM5_IRQHandler
DEFAULT_ISR_HANDLER SPI3_IRQHandler
DEFAULT_ISR_HANDLER UART4_IRQHandler
DEFAULT_ISR_HANDLER UART5_IRQHandler
DEFAULT_ISR_HANDLER TIM6_IRQHandler
DEFAULT_ISR_HANDLER TIM7_IRQHandler
DEFAULT_ISR_HANDLER DMA2_Channel1_IRQHandler
DEFAULT_ISR_HANDLER DMA2_Channel2_IRQHandler
DEFAULT_ISR_HANDLER DMA2_Channel3_IRQHandler
#ifdef __TARGET_CL
DEFAULT_ISR_HANDLER DMA2_Channel4_IRQHandler
DEFAULT_ISR_HANDLER DMA2_Channel5_IRQHandler
DEFAULT_ISR_HANDLER ETH_IRQHandler
DEFAULT_ISR_HANDLER ETH_WKUP_IRQHandler
DEFAULT_ISR_HANDLER CAN2_TX_IRQHandler
DEFAULT_ISR_HANDLER CAN2_RX0_IRQHandler
DEFAULT_ISR_HANDLER CAN2_RX1_IRQHandler
DEFAULT_ISR_HANDLER CAN2_SCE_IRQHandler
DEFAULT_ISR_HANDLER OTG_FS_IRQHandler
#else
DEFAULT_ISR_HANDLER DMA2_Channel4_5_IRQHandler
#endif

#ifndef STARTUP_FROM_RESET
DEFAULT_ISR_HANDLER reset_wait
#endif /* STARTUP_FROM_RESET */
