/****************************************************************************
*  This file is part of the lpc2000 port for the TNKernel RTOS
*
*  Copyright (c) 2009 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************
*  History:
*
*  20.09.2009  mifi   First Version
****************************************************************************/
#if defined(TN_ARCH_ARM) && defined(TN_PORT_LPC2000)

/*=========================================================================*/
/*  Includes                                                               */
/*=========================================================================*/
#include <targets/lpc2000.h>
#include "tn.h"

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/
typedef void (*IRQ_FUNC)(void);

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/
static unsigned long *VectorAddressTable = (unsigned long*)(VIC_BASE + VICVectAddr0_OFFSET);
static unsigned long *VectorControlTable = (unsigned long*)(VIC_BASE + VICVectCntl0_OFFSET);

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/
/***************************************************************************/
/*  DefaultVICHandler                                                      */
/*                                                                         */
/*  This handler is set to deal with spurious interrupt.                   */
/*  When an IRQ service routine reads the Vector Address register          */
/*  (VICVectAddr), and no IRQ slot responds as described above,            */
/*  this function will be called.                                          */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void DefaultVICHandler (void)
{
   /* 
    * If the IRQ is not installed into the VIC, and interrupt occurs, the
    * default interrupt VIC address will be used. This could happen in a 
    * race condition. For debugging, use this endless loop to trace back. 
    *
    * For more details, see Philips appnote AN10414 
    */
   VICVectAddr = 0;  /* Acknowledge Interrupt */ 
   
   while ( 1 )
   {
   }
} /* DefaultVICHandler */

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/

/***************************************************************************/
/*  tn_cpu_irq_handler                                                     */
/*                                                                         */
/*  This function will execute the user interrupt function.                */
/*                                                                         */                                                                       
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void tn_cpu_irq_handler (void)
{
   IRQ_FUNC IrqFunc;
   
   IrqFunc = (IRQ_FUNC)VICVectAddr;   
   if (IrqFunc != (IRQ_FUNC)0)
   {
      IrqFunc();
   }

   /* Acknowledge Interrupt */
   VICVectAddr = 0;
} /* tn_cpu_irq_handler */

/***************************************************************************/
/*  tn_irq_init                                                            */
/*                                                                         */
/*  Here the irq system of the cpu will be init.                           */
/*  E.g. a spurious interrupt will be set.                                 */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void tn_irq_init (void)
{
   int i;
   
   /* Disable both IRQ & FIR */
   tn_arm_disable_interrupts();
   
   /* Disable all interrupts first */
   VICIntEnClr = 0xffffffff;
   
   /* Acknowledge Interrupt */
   VICVectAddr = 0;
   
   /* Select all interrupts for IRQ, instead FIQ */
   VICIntSelect = 0;
   
   /* Set VectorAddress and VectorControl to 0 */
   for(i=0; i<TN_MAX_IRQ_PRIO; i++)
   {
      VectorAddressTable[i] = 0;
      VectorControlTable[i] = 0;
   }

   /* Install the default VIC handler */
   VICDefVectAddr = (unsigned long)DefaultVICHandler;   
} /* tn_irq_init */

/***************************************************************************/
/*  tn_irq_install                                                         */
/*                                                                         */
/*  In    : id, prio, interrupt function                                   */
/*  Out   : none                                                           */
/*  Return: TERR_NO_ERR / TERR_WRONG_PARAM                                 */
/***************************************************************************/
int tn_irq_install (TN_IRQ_ID   id,
                    TN_IRQ_PRIO prio,
                    void (*irq_func)(void))
{
   int rc = TERR_NO_ERR;
   
   if( (id >= TN_MAX_IRQ_ID) || (prio >= TN_MAX_IRQ_PRIO) )
   {
      rc = TERR_WRONG_PARAM;
   }
   else
   {
      /* Disable interrupt first */
      VICIntEnClr = (1L << id);
   
      VectorAddressTable[prio] = (unsigned long)irq_func;
      VectorControlTable[prio] = (unsigned long)(0x20 | id);
   }
   
   return(rc);
} /* tn_irq_install */

#endif /* TN_ARCH_ARM && TN_PORT_LPC2000 */
/*** EOF ***/
